#!/usr/bin/env python
"""
Simple ncurses pad scrolling example.

Example usages: cpager.py -l {1..100}
                cpager.py -f ~/path/to/file
                cpager.py -s "some super long string with\n newlines..."

Use k/j/h/l to go up/down/left/right
f/b to go to next/previous pages
g/G to go to beginning/end
q to quit
"""

import cgitb
import curses
import curses.ascii
import re
import os

from argparse import ArgumentParser, ArgumentTypeError
from cansi import Cansi
from .keys import Keys

os.environ.setdefault("ESCDELAY", "12")  # otherwise it takes an age!

# better debugging
cgitb.enable(format="text")


def chkfile(path):
    """
    Check if file exists, otherwise raise argparse exception.
    """
    if os.path.isfile(os.path.expanduser(path)):
        return os.path.abspath(os.path.expanduser(path))

    raise ArgumentTypeError(f"{path} does not exist.")


def get_args():
    """
    Return CLI arguments.
    """
    parser = ArgumentParser(description=f"Curses Pager Experiments")
    parser.add_argument("-f", "--file", type=chkfile, help="file to page")
    parser.add_argument("-l", "--list", nargs="+", help="list to page")
    parser.add_argument("-s", "--string", help=f"long string to page")
    return parser.parse_args()


def event_loop(stdscr, lines):  # pylint: disable=too-many-branches
    """
    Draw screen and wait for input.
    """
    page = Keys(stdscr, lines)
    cansi = Cansi(page.pad)
    footer = "Press [?] to view keybindings"
    while True:
        page.draw_footer(footer)
        # draw lines
        for idx, line in enumerate(lines):
            cansi.addstr(idx, 0, line)
            # if idx in page.matches:
            #     page.pad.chgat(idx, 0, curses.color_pair(100) | curses.A_BOLD)

        page.refresh()

        # react to input
        key = stdscr.getch()
        out = page.action(key)
        if out == "quit":
            break

        if out:
            footer = out


def parse_newlines(lines):
    for l in lines:
        if any(c in l for c in [os.linesep, "\n", "\\n"]):
            idx = lines.index(l)
            new_elements = re.split(f"{os.linesep}|\\\\n", l)
            lines.remove(l)
            for e in new_elements:
                lines.insert(idx, e)
                idx += 1

    return lines


def pager(lines):
    curses.wrapper(event_loop, lines)


def main():
    args = get_args()

    if args.file:
        path = os.path.abspath(os.path.expanduser(args.file))
        with open(path, "r") as file:
            lines = file.readlines()
    elif args.list:
        lines = parse_newlines(args.list)
    elif args.string:
        lines = re.split(f"{os.linesep}|\\\\n", lines)

    pager(lines)


if __name__ == "__main__":
    main()



import curses
import os
import re
from curses.textpad import Textbox
from fnmatch import fnmatch


class Action:
    def __init__(self, stdscr, lines):
        self.stdscr = stdscr
        self.lines = lines
        self.maxy, self.maxx = self.stdscr.getmaxyx()
        self.longest = len(max(self.lines, key=len))
        self.foot = curses.newwin(0, self.maxx, self.maxy - 1, 0)
        self.pad = curses.newpad(len(self.lines) + 1, self.longest + 1)
        self.pad.keypad(True)  # use function keys
        self.pad.idlok(True)
        self.pad.scrollok(True)
        self.pminrow = 0  # pad row to start displaying contents at
        self.pmincol = 0  # pad column to start displaying contents at
        self.matches = []
        curses.curs_set(0)  # hide the cursor

    def up(self):
        if self.pminrow > 0:
            self.pminrow -= 1

    def down(self):
        if self.pminrow < len(self.lines) - self.maxy:
            self.pminrow += 1

    def left(self):
        if self.pmincol > 0:
            self.pmincol -= 1

    def right(self):
        if self.pmincol < self.longest - self.maxx:
            self.pmincol += 1

    def pgup(self):
        if self.pminrow > 0:
            self.pminrow -= self.maxy

    def pgdn(self):
        if self.pminrow < len(self.lines) - self.maxy:
            self.pminrow += self.maxy
        if self.pminrow > len(self.lines) - self.maxy:
            self.pminrow = len(self.lines) - self.maxy

    def top(self):
        self.pminrow = 0

    def bottom(self):
        self.pminrow = len(self.lines) - self.maxy

    def next(self):
        if self.matches:
            for i in self.matches:
                if self.pminrow < i:
                    self.pminrow = i
                    return
            self.top()
            self.next()

    def prev(self):
        if self.matches:
            for i in reversed(self.matches):
                if self.pminrow > i:
                    self.pminrow = i
                    return
            self.bottom()
            self.prev()

    def _match(self, msg):
        search = self._textbox(msg).strip()
        if not search:
            return
        for index, line in enumerate(self.lines):
            if search in line:
                if index not in self.matches:
                    self.matches.append(index)
        if self.matches:
            self.next()

    def find(self):
        self.matches = []
        self._match("Find: ")

    def reset(self):
        self.matches = []

    def refresh(self):
        self.stdscr.noutrefresh()
        self.pad.noutrefresh(
            self.pminrow, self.pmincol, 0, 0, self.maxy - 2, self.maxx - 1
        )
        self.foot.noutrefresh()
        curses.doupdate()

    def resize(self):
        self.stdscr.erase()
        self.pad.erase()
        self.maxy, self.maxx = self.stdscr.getmaxyx()
        self.foot.mvwin(self.maxy - 1, 0)
        self.foot.resize(1, self.maxx)

    def draw_footer(self, msg):
        try:
            curses.init_pair(100, curses.COLOR_WHITE, curses.COLOR_BLUE)
            self.foot.addstr(0, 0, msg, curses.color_pair(100) | curses.A_BOLD)
            self.foot.bkgdset(curses.color_pair(100))
            self.foot.clrtoeol()  # more frugal than erase. no flicker.
        except curses.error:
            pass

    def _textbox(self, prompt):
        self.foot.erase()
        self.foot.addstr(0, 0, prompt, curses.A_BOLD)
        curses.curs_set(1)
        self.foot.refresh()
        tb = self.foot.subwin(self.maxy - 1, len(prompt))
        box = Textbox(tb)
        box.edit()
        curses.curs_set(0)
        result = box.gather()
        self.foot.erase()
        return result

    def save(self):
        path = self._textbox("Save to: ").strip()
        path = os.path.abspath(os.path.expanduser(path))
        out = None
        try:
            # removes need to use f.close
            with open(path, "a+") as f:
                for line in self.lines:
                    f.write(line)
        except FileNotFoundError:
            out = "Can't find " + path
        except IsADirectoryError:
            out = path + " is a directory."
        except UnboundLocalError:  # bit of a hack but fuck it
            out = "Not saving items."
        except Exception:
            out = "Something went wrong..."
        else:
            out = "Saved items to " + path
        finally:
            return out

    def quit(self):
        return "quit"



import curses
from .action import Action
from os import environ

environ.setdefault("ESCDELAY", "12")  # otherwise it takes an age!


class Keys(Action):
    """
    Translate keys to actions.
    """

    def __init__(self, stdscr, lines):
        super().__init__(stdscr, lines)
        self.help = [
            "[h][LEFT]     : Scroll left one character.",
            "[l][RIGHT]    : Scroll right one column.",
            "[k][UP]       : Scroll up one line.",
            "[j][DOWN]     : Scroll down one line.",
            "[f][PGDN]     : Scroll down a page of lines.",
            "[b][PGUP]     : Scroll up a page of lines.",
            "[g][HOME]     : Go to first page.",
            "[G][END]      : Go to last page.",
            "[/]           : Search via wildcards or regex.",
            "[n]           : Jump to next search result.",
            "[p]           : Jump to previous search result.",
            "[r]           : Reset search results.",
            "[?][F1]       : View this help page.",
            "[w][CTRL-s]   : Save contents to file.",
            "[q][ESC]      : Quit and exit.",
        ]

        # The keys in this dictionary map onto methods in the Action class.
        self.keys = {
            "resize": [curses.KEY_RESIZE],
            "left": [ord("h"), curses.KEY_LEFT],
            "right": [ord("l"), curses.KEY_RIGHT],
            "down": [ord("j"), curses.KEY_DOWN],
            "up": [ord("k"), curses.KEY_UP],
            "pgdn": [ord("f"), curses.KEY_NPAGE],
            "pgup": [ord("b"), curses.KEY_PPAGE],
            "top": [ord("g"), curses.KEY_HOME],
            "bottom": [ord("G"), curses.KEY_END],
            "find": [ord("/")],
            "next": [ord("n")],
            "prev": [ord("p")],
            "reset": [ord("r")],
            "show_help": [ord("?"), curses.KEY_F1],
            "save": [ord("w"), curses.ascii.ctrl(ord("s"))],
            "quit": [ord("q"), curses.ascii.ESC],
        }

    def show_help(self):
        pminrow, pmincol = self.pminrow, self.pmincol
        self.pminrow, self.pmincol = (0,) * 2
        self.pad.erase()
        while True:
            self.draw_footer("Press [q] or [ESC] to return to pager.")
            for idx, line in enumerate(self.help):
                self.pad.addstr(idx, 0, line)

            self.refresh()

            key = self.stdscr.getch()
            if self.action(key) == "quit":
                break

        self.pminrow, self.pmincol = pminrow, pmincol

    def action(self, key):
        """
        Run a key from the rowkeys dictionary as a method.
        """
        try:
            action = [k for (k, v) in self.keys.items() if key in v][0]
            return getattr(self, action)()
        except IndexError:
            pass
